#include "CRegistry.h"
//---------------------------------------------------------------------------

CRegistry::CRegistry()
{
   m_PathOpened = FALSE;
   SetLastError( TEXT("") );
   lstrcpy(m_szPath, "");
}
//---------------------------------------------------------------------------

CRegistry::CRegistry(HKEY hRootKey)
{
   m_PathOpened = FALSE;
   SetLastError( TEXT("") );
   lstrcpy(m_szPath, "");

   SetRoot(hRootKey);
}
//---------------------------------------------------------------------------

CRegistry::~CRegistry()
{
   if( m_PathOpened )
      RegCloseKey(m_hSubKey);
}
//---------------------------------------------------------------------------

BOOL CRegistry::SetRoot(HKEY hRootKey)
{
   m_hRootKey = hRootKey;
   return TRUE;
}
//---------------------------------------------------------------------------

BOOL CRegistry::OpenPath(LPTSTR szPath, BOOL bCreate)
{
   if( lstrlen(szPath) > 2047 )
   {
      SetLastError( TEXT("String too long. Path can only consist of at most 1023 characters") );
      return FALSE;
   }

   lstrcpy(m_szPath, szPath);

   if( !PathExists(m_szPath) )
   {
      if( bCreate )
      {
         if( !CreatePath(m_szPath) )
            return FALSE;
      }
      else
         return TRUE;
   }

   if( RegOpenKeyEx(m_hRootKey, m_szPath, 0, KEY_ALL_ACCESS, &m_hSubKey) != ERROR_SUCCESS )
   {
      SetLastError( TEXT("A path could not be opened") );
      return FALSE;
   }

   m_PathOpened = TRUE;
   return TRUE;
}
//---------------------------------------------------------------------------

BOOL CRegistry::ClosePath()
{
   if( RegCloseKey(m_hSubKey) != ERROR_SUCCESS )
   {
      SetLastError( TEXT("Path could not be closed") );
      return FALSE;
   }

   m_PathOpened = FALSE;
   return TRUE;
}
//---------------------------------------------------------------------------

BOOL CRegistry::PathExists(LPTSTR szPath)
{
   HKEY hKey;
   RegOpenKeyEx(m_hRootKey, szPath, 0, KEY_ALL_ACCESS, &hKey);
   if( hKey )
      RegCloseKey(hKey);
   return (BOOL)hKey;
}
//---------------------------------------------------------------------------

LPTSTR CRegistry::GetLastError()
{
   return m_LastError;
}
//---------------------------------------------------------------------------

BOOL CRegistry::CreatePath(LPTSTR szPath)
{
   DWORD dw = 0;
   if( RegCreateKeyEx(m_hRootKey,
                      szPath,
                      0,
                      NULL,
                      REG_OPTION_NON_VOLATILE,
                      KEY_ALL_ACCESS,
                      NULL,
                      &m_hSubKey,
                      &dw
                     ) != ERROR_SUCCESS )
   {
      SetLastError( TEXT("Path could not be created") );
      return FALSE;
   }

   return TRUE;
}
//---------------------------------------------------------------------------

BOOL CRegistry::DeleteLastPathKey(LPTSTR szPath)
{
   if( szPath!=NULL )
   {
      SetLastError( TEXT("Parameter may not be NULL") );
      return FALSE;
   }

   if( RegDeleteKey(m_hRootKey, szPath) != ERROR_SUCCESS )
   {
      SetLastError( TEXT("Key could not be deleted") );
      return FALSE;
   }

   return TRUE;
}
//---------------------------------------------------------------------------

BOOL CRegistry::ValueExists(LPTSTR szValName)
{
   DWORD dw;
   return (RegQueryValueEx(m_hSubKey, szValName, NULL, NULL, NULL, &dw) == ERROR_SUCCESS);
}
//---------------------------------------------------------------------------

BOOL CRegistry::WriteValue(LPTSTR szValName, PVOID value, int bufSize, DWORD dwType)
{
   if( !m_PathOpened )
   {
      SetLastError( TEXT("Path not specified yet") );
      return FALSE;
   }

   if( RegSetValueEx(m_hSubKey,
                     szValName,
                     0,
                     dwType,
                     (BYTE*)value,
                     (DWORD)bufSize) != ERROR_SUCCESS )
   {
      SetLastError( TEXT("Value could not be written") );
      return FALSE;
   }

   return TRUE;
}
//---------------------------------------------------------------------------

BOOL CRegistry::WriteString(LPTSTR szValName, LPTSTR szValue)
{
   return WriteValue(szValName, szValue, lstrlen(szValue) + 1, REG_SZ);
}
//---------------------------------------------------------------------------

BOOL CRegistry::WriteDWORD(LPTSTR szValName, DWORD dwValue)
{
   return WriteValue(szValName, &dwValue, sizeof(DWORD), REG_DWORD);
}
//---------------------------------------------------------------------------

BOOL CRegistry::WriteBinary(LPTSTR szValName, LPBYTE lpValue, DWORD dwNumBytes)
{
   return WriteValue(szValName, lpValue, dwNumBytes, REG_BINARY);
}
//---------------------------------------------------------------------------

BOOL CRegistry::QueryValue(LPTSTR szValName, LPDWORD lpdwType, LPVOID lpValue,
                           LPDWORD lpdwBufSize)
{
   if( !ValueExists(szValName) )
   {
      SetLastError( TEXT("No such value exists") );
      return FALSE;
   }
   
   if( RegQueryValueEx(m_hSubKey,
                       szValName,
                       NULL,
                       lpdwType,
                       (LPBYTE)lpValue,
                       lpdwBufSize) != ERROR_SUCCESS )
   {
      SetLastError( TEXT("A value could not be read") );
      return FALSE;
   }

   return TRUE;
}
//---------------------------------------------------------------------------

BOOL CRegistry::GetValueSize(LPTSTR szValName, LPDWORD lpdwSize)
{
   return QueryValue(szValName, NULL, NULL, lpdwSize);
}
//---------------------------------------------------------------------------

BOOL CRegistry::GetValueType(LPTSTR szValName, LPDWORD lpdwType)
{
   DWORD dw = 0;
   return QueryValue(szValName, lpdwType, NULL, &dw);
}
//---------------------------------------------------------------------------

BOOL CRegistry::ReadString(LPTSTR szValName, LPTSTR lpszRetVal, DWORD dwBufSize)
{
   DWORD dw = dwBufSize;
   return QueryValue(szValName, NULL, lpszRetVal, &dw);
}
//---------------------------------------------------------------------------

BOOL CRegistry::ReadDWORD(LPTSTR szValName, LPDWORD lpdwRetVal)
{
   DWORD dw = sizeof(DWORD);
   return QueryValue(szValName, NULL, lpdwRetVal, &dw);
}
//---------------------------------------------------------------------------

BOOL CRegistry::ReadBinary(LPTSTR szValName, LPBYTE lpRetVal, DWORD dwBufSize)
{
   DWORD dw = dwBufSize;
   return QueryValue(szValName, NULL, lpRetVal, &dw);
}
//---------------------------------------------------------------------------

VOID CRegistry::SetLastError(LPCTSTR szLastError)
{
   lstrcpy(m_LastError, szLastError);
}
//---------------------------------------------------------------------------
