//---------------------------------------------------------------------------
#ifndef CRegistryH
#define CRegistryH
//---------------------------------------------------------------------------
#include <windows.h>
//---------------------------------------------------------------------------

// Possible types for values:
//--------------------------------
// REG_BINARY	            Binary data in any form.
// REG_DWORD	            A 32-bit number.
// REG_DWORD_LITTLE_ENDIAN  A 32-bit number in little-endian format (same as REG_DWORD). In little-endian format, the most significant byte of a word is the high-order byte. This is the most common format for computers running Windows NT and Windows 95.
// REG_DWORD_BIG_ENDIAN	    A 32-bit number in big-endian format. In big-endian format, the most significant byte of a word is the low-order byte.
// REG_EXPAND_SZ	    A null-terminated string that contains unexpanded references to environment variables (for example, %PATH%). It will be a Unicode or ANSI string depending on whether you use the Unicode or ANSI functions.
// REG_LINK	            A Unicode symbolic link.
// REG_MULTI_SZ	            An array of null-terminated strings, terminated by two null characters.
// REG_RESOURCE_LIST	    A device-driver resource list.
// REG_SZ	            A null-terminated string. It will be a Unicode or ANSI string depending on whether you use the Unicode or ANSI functions.

// Notes on the write-functions:
//-----------------------------------
// - they overwrite any value careless of which type it may be
// - they act as creating functions if the specified value does not exist
//   therefore there is no need for functions to create values

class CRegistry
{
private:
    HKEY  m_hRootKey;
    HKEY  m_hSubKey;
    TCHAR m_szPath[2048];
    TCHAR m_LastError[1024];
    BOOL  m_PathOpened;

    VOID  SetLastError(LPCTSTR szLastError);
    BOOL  WriteValue(LPTSTR szValName, PVOID value, int bufSize, DWORD dwType);
    BOOL  QueryValue(LPTSTR szValName, LPDWORD lpdwType, LPVOID lpValue, LPDWORD lpdwBufSize);

public:
    CRegistry();
    CRegistry(HKEY hRootKey);
    ~CRegistry();
    BOOL SetRoot(HKEY hRootKey);
    // Path functions
    BOOL OpenPath(LPTSTR szPath, BOOL bCreate);
    BOOL ClosePath();
    BOOL PathExists(LPTSTR szPath);
    BOOL CreatePath(LPTSTR szPath);
    BOOL DeleteLastPathKey(LPTSTR szPath); // Deletes last key in szPath
    // Value functions
    BOOL ValueExists(LPTSTR szValName);
    BOOL GetValueSize(LPTSTR szValName, LPDWORD lpdwSize);
    BOOL GetValueType(LPTSTR szValName, LPDWORD lpdwType);
    BOOL WriteString(LPTSTR szValName, LPTSTR szValue);
    BOOL WriteDWORD(LPTSTR szValName, DWORD dwValue);
    BOOL WriteBinary(LPTSTR szValName, LPBYTE lpValue, DWORD dwNumBytes);
    BOOL ReadString(LPTSTR szValName, LPTSTR lpszRetVal, DWORD dwBufSize);
    BOOL ReadDWORD(LPTSTR szValName, LPDWORD lpdwRetVal);
    BOOL ReadBinary(LPTSTR szValName, LPBYTE lpRetVal, DWORD dwBufSize);
    // Other functions
    LPTSTR GetLastError();
};
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
 